@@ -29,6 +29,8 @@
 #include "cfgfile.h"
 #include "utils.h"
 #include "center.h"
+#include "resource.h"
+#include "cfgfile.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -131,7 +133,7 @@ struct log_source_driver *make_affile_source(const char *name, int flags)
   
 	self->super.super.super.init = do_init_affile_source;
 	self->super.super.super.handler = do_affile_handle_line;
-	self->name = c_format_cstring("%z%c", name, 0);
+	self->name = c_format_cstring("%z", name);
 	self->flags = flags;
 	return &self->super;
 }
@@ -141,6 +143,7 @@ struct log_source_driver *make_affile_source(const char *name, int flags)
        (name affile_dest_reaper)
        (super callback)
        (vars
+         (cfg object syslog_config)
          (affile_dest object affile_dest_writer)))
 */
 
@@ -157,16 +160,19 @@ static void do_reap_affile_dest(struct callback *c)
 	if (self->affile_dest->next) {
 		self->affile_dest->next->prev = self->affile_dest->prev;
 	}
-	if (self->affile_dest && self->affile_dest->dest)
-		close_fd(&self->affile_dest->dest->super, 0);
+	if (self->affile_dest && self->affile_dest->dest) {
+		KILL_RESOURCE_NODE(self->cfg->resources, self->affile_dest->res);
+	}
 }
 
 static struct callback *
-make_affile_dest_reaper(struct affile_dest_writer *affile_dest)
+make_affile_dest_reaper(struct syslog_config *cfg,
+                        struct affile_dest_writer *affile_dest)
 {
 	NEW(affile_dest_reaper, self);
 	
 	self->super.f = do_reap_affile_dest;
+	self->cfg = cfg;
 	self->affile_dest = affile_dest;
 	return &self->super;
 }
@@ -176,6 +182,8 @@ make_affile_dest_reaper(struct affile_dest_writer *affile_dest)
        (name affile_dest_writer)
        (super log_handler)
        (vars
+         (res pointer "struct resource_node")
+         (time_reap simple UINT32)
          (owner object affile_dest)
          (dest object io_fd)
 	 (reap simple "struct callout *")
@@ -205,9 +213,10 @@ do_init_dest_writer(struct log_handler *c, struct syslog_config *cfg, struct per
 				                     MAX(self->owner->super.log_fifo_size + 1, 
 					                 self->owner->sync_freq)),
 				 NULL);
-		REMEMBER_RESOURCE(cfg->resources, &self->dest->super.super);
+		self->res = REMEMBER_RESOURCE(cfg->resources, &self->dest->super.super);
+		self->time_reap = cfg->time_reap;
 		if ((self->owner->flags & AFFILE_NO_EXPAND) == 0)
-			self->reap = io_callout(cfg->backend, TIME_REAP_DEST, make_affile_dest_reaper(self));
+			self->reap = io_callout(cfg->backend, cfg->time_reap, make_affile_dest_reaper(cfg, self));
 		/* self->attempts = 0; */
 	}
 	else {
@@ -225,7 +234,7 @@ static void do_handle_dest_writer(struct log_handler *c, struct log_info *msg)
 	if (self->dest) {
 		A_WRITE_STRING(&self->dest->buffer->super, c_format("%S %S %S\n", msg->date, msg->host, msg->msg));
 		if (self->reap)
-			io_callout_set_timeout(self->reap, TIME_REAP_DEST);
+			io_callout_set_timeout(self->reap, self->time_reap);
 	}
 	log_info_free(msg);
 }
@@ -360,7 +369,7 @@ struct macro_def {
 	int id;
 };
 
-void expand_macro(int id, char **dest, int *left, struct log_info *msg)
+void expand_macro(struct syslog_config *cfg, int id, char **dest, int *left, struct log_info *msg)
 {
 	int length = 0;
 	
@@ -419,8 +428,16 @@ void expand_macro(int id, char **dest, int *left, struct log_info *msg)
 	}
 	case 'y':
 	case 'm':
-	case 'd': {
-		struct tm *tm = localtime(&msg->stamp);
+	case 'd': 
+	case 'H':
+	case 'M':
+	case 'S': {
+		struct tm *tm;
+
+		if (cfg->use_time_recvd)
+			tm = localtime(&msg->recvd);
+		else
+			tm = localtime(&msg->stamp);
 
 		switch (id) {
 		case 'y':
@@ -432,6 +449,15 @@ void expand_macro(int id, char **dest, int *left, struct log_info *msg)
 		case 'd':
 			length = snprintf(*dest, *left, "%02d", tm->tm_mday);
 			break;
+		case 'H':
+			length = snprintf(*dest, *left, "%02d", tm->tm_hour);
+			break;
+		case 'M':
+			length = snprintf(*dest, *left, "%02d", tm->tm_min);
+			break;
+		case 'S':
+			length = snprintf(*dest, *left, "%02d", tm->tm_sec);
+			break;
 		}
 		break;
 	}
@@ -443,7 +469,7 @@ void expand_macro(int id, char **dest, int *left, struct log_info *msg)
 }
 
 struct ol_string *
-expand_macros(struct ol_string *template, struct log_info *msg)
+expand_macros(struct syslog_config *cfg, struct ol_string *template, struct log_info *msg)
 {
 	static struct macro_def macros[] = {
 		{ "HOST", 'h' },
@@ -453,7 +479,10 @@ expand_macros(struct ol_string *template, struct log_info *msg)
 		{ "PROGRAM", 'p' },
 		{ "YEAR", 'y' },
 		{ "MONTH", 'm' },
-		{ "DAY", 'd' }
+		{ "DAY", 'd' },
+		{ "HOUR", 'H' },
+		{ "MIN", 'M' },
+		{ "SEC", 'S' }
 	};
 	char format[MAX_EXPANDED_MACRO], *format_ptr = format;
 	int left = sizeof(format);
@@ -478,7 +507,7 @@ expand_macros(struct ol_string *template, struct log_info *msg)
 			}
 			else {
 				i += strlen(macros[j].name) + 1;
-				expand_macro(macros[j].id, &format_ptr, &left, msg);
+				expand_macro(cfg, macros[j].id, &format_ptr, &left, msg);
 			}
 		}
 		else {
@@ -509,7 +538,7 @@ affile_find_writer(struct affile_dest *self,
 			return NULL;
 	}
 
-	expanded_fname = expand_macros(self->template_fname, msg);
+	expanded_fname = expand_macros(self->cfg, self->template_fname, msg);
 	p = self->writers;
 	while (p && 
 	       ((p->expanded_fname->length != expanded_fname->length) || 
@@ -547,6 +576,19 @@ do_init_affile_dest(struct log_handler *c,
 		self->super.log_fifo_size = cfg->log_fifo_size;
 	if (cfg->create_dirs)
 		self->flags |= AFFILE_CREATE_DIRS;
+	if (self->uid == -1)
+		self->uid = cfg->uid;
+	if (self->gid == -1)
+		self->gid = cfg->gid;
+	if (self->perm == -1)
+		self->perm = cfg->perm;
+	if (self->dir_uid == -1)
+		self->dir_uid = cfg->dir_uid;
+	if (self->dir_gid == -1)
+		self->dir_gid = cfg->dir_gid;
+	if (self->dir_perm == -1)
+		self->dir_perm = cfg->dir_perm;
+	
 	self->cfg = cfg;
 	return ST_OK | ST_GOON;
 }
@@ -572,8 +614,10 @@ struct log_dest_driver *make_affile_dest(const char *name, int flags)
 	self->super.super.init = do_init_affile_dest;
 	self->super.super.handler = do_handle_affile_log;
 	self->super.log_fifo_size = -1;
-	self->perm = 0600;
-	self->dir_perm = 0700;
+	self->uid = self->gid = -1;
+	self->perm = -1;
+	self->dir_uid = self->dir_gid = -1;
+	self->dir_perm = -1;
 	self->template_fname = c_format_cstring("%z", name);
 	if (strchr(name, '$') == NULL)
 		flags |= AFFILE_NO_EXPAND;
