@@ -28,6 +28,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
+#include <pwd.h>
+#include <grp.h>
 
 #define CLASS_DEFINE
 #include "cfgfile.h.x"
@@ -62,6 +64,60 @@ void add_log_connection(struct log_connection *conn)
 	configuration->connections = conn;
 }
 
+void cfg_set_owner(char *uid)
+{
+	struct passwd *pwd;
+
+	pwd = getpwnam(uid);
+	if (pwd)
+		configuration->uid = pwd->pw_uid;
+	else
+		configuration->uid = atoi(uid);
+}
+
+void cfg_set_group(char *gid)
+{
+	struct group *grp;
+
+	grp = getgrnam(gid);
+	if (grp)
+		configuration->gid = grp->gr_gid;
+	else
+		configuration->gid = atoi(gid);
+}
+
+void cfg_set_perm(int perm)
+{
+	configuration->perm = perm;
+}
+
+void cfg_set_dir_owner(char *uid)
+{
+	struct passwd *pwd;
+
+	pwd = getpwnam(uid);
+	if (pwd)
+		configuration->dir_uid = pwd->pw_uid;
+	else
+		configuration->dir_uid = atoi(uid);
+}
+
+void cfg_set_dir_group(char *gid)
+{
+	struct group *grp;
+
+	grp = getgrnam(gid);
+	if (grp)
+		configuration->dir_gid = grp->gr_gid;
+	else
+		configuration->dir_gid = atoi(gid);
+}
+
+void cfg_set_dir_perm(int perm)
+{
+	configuration->dir_perm = perm;
+}
+
 struct persistent_info *
 make_persistent_info(struct ol_string *name, 
 		     struct ol_object *o, 
@@ -237,9 +293,16 @@ struct syslog_config *make_syslog_config(const char *name, struct io_backend *ba
 	self->use_fqdn = 0;
 	self->use_dns = 1;
 	self->time_reopen = 60;
+	self->time_reap = 60;
 	self->log_fifo_size = 100;
 	self->gc_idle_threshold = 100;
 	self->gc_busy_threshold = 3000;
+	self->uid = 0;
+	self->gid = 0;
+	self->perm = 0600;
+	self->dir_uid = 0;
+	self->dir_gid = 0;
+	self->dir_perm = 0700;
 	if ((cfg = fopen(name, "r")) != NULL) {
 		lex_init(cfg);
 		res = yyparse();
