@@ -50,10 +50,18 @@
 	(mark_freq simple UINT32)
 	(sync_freq simple UINT32)
 	(time_reopen simple UINT32)
+	(time_reap simple UINT32)
 	(log_fifo_size simple UINT32)
+	(use_time_recvd simple UINT32)
 	(use_fqdn simple UINT32)
 	(use_dns simple UINT32)
 	(create_dirs simple UINT32)
+	(uid simple int)
+	(gid simple int)
+	(perm simple int)
+	(dir_uid simple int)
+	(dir_gid simple int)
+	(dir_perm simple int)
 	(gc_busy_threshold simple UINT32)
 	(gc_idle_threshold simple UINT32)
 	; internal messages should come from this source
@@ -105,6 +113,17 @@ void add_dest_group(struct log_dest_group *grp);
 void add_source_group(struct log_source_group *grp);
 void add_log_connection(struct log_connection *conn);
 void add_filter_rule(struct log_filter *filter);
+void cfg_set_owner(char *uid);
+void cfg_set_group(char *gid);
+void cfg_set_perm(int perm);
+void cfg_set_dir_owner(char *uid);
+void cfg_set_dir_group(char *gid);
+void cfg_set_dir_perm(int perm);
+
+
+
+
+
 
 struct syslog_config *make_syslog_config(const char *name, struct io_backend *backend);
 
